function [x,resvec] = mycg(A,b,x0,tol,maxit)
% Gradient descent method for the solution of a linear system
% Inputs:    A - system matrix
%            b - right hand side
%           x0 - initial guess
%          tol - method stops if the maximum norm of the residual
%                b-A*x is less than tol   
%        maxit - maximum number of iterations
% 
% Outputs:   x - solution vector
%       resvec - residual vector

% initial residual  
  res = b-A*x0;
  normres = norm(b-A*x0); %2-norm
  
% save residual  
  resvec = zeros(maxit+1,1);
  resvec(1) = normres;

  x = x0; 
  cnt = 1;
  
  p = res;
  a = res'*res;
  while normres > tol && cnt <= maxit
%##########################################################################      
    Ap = A*p;  
    lambda = a/(p'*Ap);
    x = x + lambda*p;
    res = res - lambda*Ap;
    a2 = a;
    a = res'*res;
    p = res + a/a2*p;
%##########################################################################      
    
    normres = norm(b-A*x); % residual
    cnt = cnt + 1; % increase counter
    resvec(cnt) = normres; % save residual
  end
  
% shorten resvec
  resvec = resvec(1:cnt);