function [x,resvec] = mysor(A,b,x0,tol,maxit,omega)
% SOR method for the solution of a linear system
% Inputs:    A - system matrix
%            b - right hand side
%           x0 - initial guess
%          tol - method stops if the maximum norm of the residual
%                b-A*x is less than tol   
%        maxit - maximum number of iterations
% 
% Outputs:   x - solution vector
%       resvec - residual vector

% initial residual  
  normres = norm(b-A*x0,'inf');
  
% save residual  
  resvec = zeros(maxit+1,1);
  resvec(1) = normres;

% SOR matrices
%##########################################################################
  D = diag(diag(A));
  L = tril(A,-1);
  R = triu(A,+1);
  invDL = inv(D+omega*L);
  
  MSOR = invDL*((1-omega)*D-omega*R);
  NSOR = omega*invDL*b;
%##########################################################################  

  x = x0; 
  cnt = 1;
  while normres > tol && cnt <= maxit
% #########################################################################      
    x =  MSOR*x + NSOR; % SOR iteration
% #########################################################################          
    normres = norm(b-A*x,'inf'); % residual
    cnt = cnt + 1; % increase counter
    resvec(cnt) = normres; % save residual
  end
  
% shorten resvec
  resvec = resvec(1:cnt);