clear, clc, close all

%% Parameters
   a = 0.5; % Advection parameter
   b = 2; % Reaction parameter
   
   tol = 1e-6; % Tolerance for stopping criterion
   
   maxit = 10000; % Maxium number of iterations
   
%% Linear system
   N = 99;
   dx = 1/(N+1);
   
   e = ones(N,1);
   Dxx = spdiags([e -2*e e],[-1 0 1],N,N); % 1D central difference
   Axx = kron(speye(N,N),Dxx)/(dx^2); % 2nd derivative with respect to x
   Ayy = kron(Dxx,speye(N,N))/(dx^2); % 2nd derivative with respect to y
   
   Dx = spdiags([-e e],[-1 1],N,N); % 1D central difference
   Ax= kron(speye(N,N),Dx)/(2*dx);
   
   I = speye(N^2);
   
   A = -Axx - Ayy -a*Ax - b*I; % Matrix   
   
   b = zeros(N^2,1); % Right hand side
  
   u0 = ones(N^2,1); % Initial guess
      
   tic;
   [xBiCG,resBiCG] = mybicg(A,b,u0,tol,maxit);
   tBiCG = toc;
      
%% Output
 % plotrestime(res1,time1,'name1,res2,time2,'name2',...)
   plotrestime(resBiCG,tBiCG,'BiCG');      
